#
# This is the SPEC file for creating binary RPMs for the Dom0.
#
#
# The Qubes OS Project, http://www.qubes-os.org
#
# Copyright (C) 2013  Marek Marczykowski  <marmarek@invisiblethingslab.com>
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
#
#

%{!?python_sitearch: %define python_sitearch %(%{__python} -c "from distutils.sysconfig import get_python_lib; print get_python_lib(1)")}

%define _dracutmoddir	/usr/lib/dracut/modules.d

Name:		qubes-core-dom0-linux
Version:	4.1.6
Release:	1%{?dist}
Summary:	Linux-specific files for Qubes dom0

Group:		Qubes
Vendor:		Invisible Things Lab
License:	GPL
URL:		http://www.qubes-os.org

BuildRequires:  ImageMagick
BuildRequires:  pandoc
BuildRequires:  qubes-utils-devel >= 3.1.3
BuildRequires:  gcc
Requires:	qubes-core-dom0
Requires:	python3-qubesadmin
Requires:	qubes-core-qrexec-dom0
Requires:	qubes-core-admin-client
Requires:	qubes-utils >= 3.1.3
Requires:	qubes-utils-libs >= 4.0.16
Conflicts:	qubes-core-dom0 < 4.0.23
Requires:	%{name}-kernel-install
Requires:	xdotool
Requires:	createrepo_c

Source0: %{name}-%{version}.tar.gz

%description
Linux customizations required to use system as Qubes dom0.
Additionally some graphical elements for every Linux desktop envirnment (icons,
appmenus etc).

%package kernel-install
Summary:	Kernel install hook for Xen-based system

# get rid of os-prober, it tries to mount and parse all the block devices in
# the system, including loop*
Provides: os-prober
Obsoletes: os-prober

%description kernel-install
Kernel install hook for Xen-based system.

%prep
%setup -q

%build
export BACKEND_VMM=xen
(cd dom0-updates; make)
(cd file-copy-vm; make)
(cd doc; make manpages)

%install

## Appmenus
install -d $RPM_BUILD_ROOT/etc/qubes-rpc/policy
cp qubesappmenus/qubes.SyncAppMenus.policy $RPM_BUILD_ROOT/etc/qubes-rpc/policy/qubes.SyncAppMenus

### Dom0 updates
install -D dom0-updates/qubes-dom0-updates.cron $RPM_BUILD_ROOT/etc/cron.daily/qubes-dom0-updates.cron
install -D dom0-updates/qubes-dom0-update $RPM_BUILD_ROOT/usr/bin/qubes-dom0-update
install -D dom0-updates/qubes-receive-updates $RPM_BUILD_ROOT/usr/libexec/qubes/qubes-receive-updates
install -D dom0-updates/patch-dnf-yum-config $RPM_BUILD_ROOT/usr/lib/qubes/patch-dnf-yum-config
install -m 0644 -D dom0-updates/qubes-cached.repo $RPM_BUILD_ROOT/etc/yum.real.repos.d/qubes-cached.repo
install -D dom0-updates/qfile-dom0-unpacker $RPM_BUILD_ROOT/usr/libexec/qubes/qfile-dom0-unpacker
ln -s ../../usr/libexec/qubes/qubes-receive-updates $RPM_BUILD_ROOT/etc/qubes-rpc/qubes.ReceiveUpdates
install -m 0664 -D dom0-updates/qubes.ReceiveUpdates.policy $RPM_BUILD_ROOT/etc/qubes-rpc/policy/qubes.ReceiveUpdates

install -d $RPM_BUILD_ROOT/var/lib/qubes/updates

# Qrexec services
mkdir -p $RPM_BUILD_ROOT/usr/lib/qubes/qubes-rpc $RPM_BUILD_ROOT/etc/qubes-rpc/policy
cp qubes-rpc/* $RPM_BUILD_ROOT/usr/lib/qubes/qubes-rpc/
for i in qubes-rpc/*; do ln -s ../../usr/lib/qubes/$i $RPM_BUILD_ROOT/etc/qubes-rpc/$(basename $i); done
cp qubes-rpc-policy/* $RPM_BUILD_ROOT/etc/qubes-rpc/policy/

### pm-utils
mkdir -p $RPM_BUILD_ROOT/usr/lib64/pm-utils/sleep.d
cp pm-utils/52qubes-pause-vms $RPM_BUILD_ROOT/usr/lib64/pm-utils/sleep.d/
mkdir -p $RPM_BUILD_ROOT/usr/lib/systemd/system
cp pm-utils/qubes-suspend.service $RPM_BUILD_ROOT/usr/lib/systemd/system/

### Dracut module
mkdir -p $RPM_BUILD_ROOT/etc/dracut.conf.d
cp dracut/dracut.conf.d/* $RPM_BUILD_ROOT/etc/dracut.conf.d/

mkdir -p $RPM_BUILD_ROOT%{_dracutmoddir}
cp -r dracut/modules.d/* $RPM_BUILD_ROOT%{_dracutmoddir}/

### Others
mkdir -p $RPM_BUILD_ROOT/etc/sysconfig
install -m 0644 -D system-config/limits-qubes.conf $RPM_BUILD_ROOT/etc/security/limits.d/99-qubes.conf
install -D system-config/cpufreq-xen.modules $RPM_BUILD_ROOT/etc/sysconfig/modules/cpufreq-xen.modules
install -m 0440 -D system-config/qubes.sudoers $RPM_BUILD_ROOT/etc/sudoers.d/qubes
install -D system-config/polkit-1-qubes-allow-all.rules $RPM_BUILD_ROOT/etc/polkit-1/rules.d/00-qubes-allow-all.rules
install -D system-config/qubes-dom0.modules $RPM_BUILD_ROOT/etc/sysconfig/modules/qubes-dom0.modules
install -D system-config/qubes-sync-clock.cron $RPM_BUILD_ROOT/etc/cron.d/qubes-sync-clock.cron
install -D system-config/lvm-cleanup.cron-daily $RPM_BUILD_ROOT/etc/cron.daily/lvm-cleanup
install -d $RPM_BUILD_ROOT/etc/udev/rules.d
install -m 644 system-config/00-qubes-ignore-devices.rules $RPM_BUILD_ROOT/etc/udev/rules.d/
install -m 644 system-config/12-qubes-ignore-lvm-devices.rules $RPM_BUILD_ROOT/etc/udev/rules.d/
install -m 644 -D system-config/disable-lesspipe.sh $RPM_BUILD_ROOT/etc/profile.d/zz-disable-lesspipe.sh
install -m 755 -D system-config/kernel-grub2.install $RPM_BUILD_ROOT/usr/lib/kernel/install.d/80-grub2.install
install -m 755 -D system-config/kernel-xen-efi.install $RPM_BUILD_ROOT/usr/lib/kernel/install.d/90-xen-efi.install
install -m 755 -D system-config/kernel-remove-bls.install $RPM_BUILD_ROOT/usr/lib/kernel/install.d/99-remove-bls.install
install -m 644 -D system-config/75-qubes-dom0.preset \
    $RPM_BUILD_ROOT/usr/lib/systemd/system-preset/75-qubes-dom0.preset
install -m 644 -D system-config/75-qubes-dom0-user.preset \
    $RPM_BUILD_ROOT/usr/lib/systemd/user-preset/75-qubes-dom0-user.preset
install -m 644 -D system-config/99-qubes-default-disable.preset \
    $RPM_BUILD_ROOT/usr/lib/systemd/system-preset/99-qubes-default-disable.preset
install -d $RPM_BUILD_ROOT/etc/dnf/protected.d
install -m 0644 system-config/dnf-protected-qubes-core-dom0.conf  \
        $RPM_BUILD_ROOT/etc/dnf/protected.d/qubes-core-dom0.conf


touch $RPM_BUILD_ROOT/var/lib/qubes/.qubes-exclude-block-devices

# file copy to VM
install -m 755 file-copy-vm/qfile-dom0-agent $RPM_BUILD_ROOT/usr/lib/qubes/
install -m 755 file-copy-vm/qvm-copy-to-vm $RPM_BUILD_ROOT/usr/bin/
install -m 755 file-copy-vm/qvm-copy $RPM_BUILD_ROOT/usr/bin/
ln -s qvm-copy-to-vm $RPM_BUILD_ROOT/usr/bin/qvm-move-to-vm
ln -s qvm-copy $RPM_BUILD_ROOT/usr/bin/qvm-move

### Documentation
(cd doc; make DESTDIR=$RPM_BUILD_ROOT install)

%pre
if ! grep -q ^qubes: /etc/group ; then
		groupadd qubes
fi

%post

/usr/lib/qubes/patch-dnf-yum-config

systemctl enable qubes-suspend.service >/dev/null 2>&1

# migrate dom0-updates check disable flag
if [ $1 -ge 2 ]; then
    if [ -e /var/lib/qubes/updates/disable-updates ]; then
        qvm-features dom0 service.qubes-update-check ''
        rm -f /var/lib/qubes/updates/disable-updates
    fi
fi

%preun
if [ "$1" = 0 ] ; then
	# no more packages left

    systemctl disable qubes-suspend.service > /dev/null 2>&1
fi

%triggerin -- PackageKit
# dom0 have no network, but still can receive updates (qubes-dom0-update)
sed -i 's/^UseNetworkHeuristic=.*/UseNetworkHeuristic=false/' /etc/PackageKit/PackageKit.conf

%triggerin -- xorg-x11-drv-vmmouse
# Remove unnecessary udev rules that causes problems in dom0 (#605)
rm -f /lib/udev/rules.d/69-xorg-vmmouse.rules

%triggerin -- grub2-tools
chmod -x /etc/grub.d/10_linux

%files
/etc/qubes-rpc/policy/qubes.SyncAppMenus
# Dom0 updates
/etc/cron.daily/qubes-dom0-updates.cron
/etc/yum.real.repos.d/qubes-cached.repo
/usr/bin/qubes-dom0-update
/usr/lib/qubes/patch-dnf-yum-config
%attr(4750,root,qubes) /usr/libexec/qubes/qfile-dom0-unpacker
/usr/libexec/qubes/qubes-receive-updates
/etc/qubes-rpc/qubes.ReceiveUpdates
%attr(0664,root,qubes) %config(noreplace) /etc/qubes-rpc/policy/qubes.ReceiveUpdates
%attr(0770,root,qubes) %dir /var/lib/qubes/updates
# Qrexec services
/etc/qubes-rpc/qubes.repos.*
/usr/lib/qubes/qubes-rpc/qubes.repos.*
%attr(0664,root,qubes) %config(noreplace) /etc/qubes-rpc/policy/qubes.repos.List
%attr(0664,root,qubes) %config(noreplace) /etc/qubes-rpc/policy/qubes.repos.Enable
%attr(0664,root,qubes) %config(noreplace) /etc/qubes-rpc/policy/qubes.repos.Disable
# Dracut module
/etc/dracut.conf.d/*
%dir %{_dracutmoddir}/90macbook12-spi-driver
%{_dracutmoddir}/90macbook12-spi-driver/*
%dir %{_dracutmoddir}/90qubes-pciback
%{_dracutmoddir}/90qubes-pciback/*
%dir %{_dracutmoddir}/90extra-modules
%{_dracutmoddir}/90extra-modules/*
# file copy
/usr/bin/qvm-copy-to-vm
/usr/bin/qvm-move-to-vm
/usr/bin/qvm-copy
/usr/bin/qvm-move
/usr/lib/qubes/qfile-dom0-agent
# pm-utils
/usr/lib64/pm-utils/sleep.d/52qubes-pause-vms
/usr/lib/systemd/system/qubes-suspend.service
# Others
/etc/sysconfig/modules/qubes-dom0.modules
/etc/sysconfig/modules/cpufreq-xen.modules
/etc/sudoers.d/qubes
/etc/polkit-1/rules.d/00-qubes-allow-all.rules
/etc/security/limits.d/99-qubes.conf
%config /etc/udev/rules.d/00-qubes-ignore-devices.rules
%config /etc/udev/rules.d/12-qubes-ignore-lvm-devices.rules
%attr(0644,root,root) /etc/cron.d/qubes-sync-clock.cron
/etc/cron.daily/lvm-cleanup
%config(noreplace) /etc/profile.d/zz-disable-lesspipe.sh
%config(noreplace) /etc/dnf/protected.d/qubes-core-dom0.conf
/usr/lib/systemd/system-preset/75-qubes-dom0.preset
/usr/lib/systemd/system-preset/99-qubes-default-disable.preset
/usr/lib/systemd/user-preset/75-qubes-dom0-user.preset
/var/lib/qubes/.qubes-exclude-block-devices
# Man
%{_mandir}/man1/qvm-*.1*
%{_mandir}/man1/qubes-*.1*

%files kernel-install
/usr/lib/kernel/install.d/80-grub2.install
/usr/lib/kernel/install.d/90-xen-efi.install
/usr/lib/kernel/install.d/99-remove-bls.install

%changelog
* Sun Jun 07 2020 Qubes OS Team <qubes-devel@googlegroups.com>
- For complete changelog see: https://github.com/QubesOS/qubes-core-admin-linux.git

* Sun Jun 07 2020 Marek Marczykowski-Górecki <marmarek@invisiblethingslab.com> - 069e2a3
- version 4.1.6

* Sat Jun 06 2020 Frédéric Pierret (fepitre) <frederic.pierret@qubes-os.org> - b95f271
- Prevent failing if cannot set 'update-available' feature

* Fri Jun 05 2020 Frédéric Pierret (fepitre) <frederic.pierret@qubes-os.org> - df3f29d
- Make ShellCheck happy and fix constant expression

* Sat May 23 2020 Frédéric Pierret (fepitre) <frederic.pierret@qubes-os.org> - 31173e5
- Fix typo in qubes-receive-updates

* Wed Mar 11 2020 Marek Marczykowski-Górecki <marmarek@invisiblethingslab.com> - 75c4156
- version 4.1.5

* Tue Mar 10 2020 hal18000 <hal18000@protonmail.com> - 1a28a1c
- Fix kernel install script clobbering kernel params ending in default=

* Thu Feb 27 2020 Andrey Arapov <andrey.arapov@nixaid.com> - 2df917e
- dracut: add roadrunner2/macbook12-spi-driver on MacBook HW

* Tue Feb 25 2020 Marek Marczykowski-Górecki <marmarek@invisiblethingslab.com> - b6f9f2c
- version 4.1.4

* Mon Feb 24 2020 Marek Marczykowski-Górecki <marmarek@invisiblethingslab.com> - e59c6b8
- Replace /etc/qubes-rpc/qubes.ReceiveUpdates with a symlink

* Fri Feb 14 2020 Paweł Marczewski <pawel@invisiblethingslab.com> - e4d73bb
- qubes-dom0-update: don't use 'script' if not in interactive mode

* Fri Feb 14 2020 Paweł Marczewski <pawel@invisiblethingslab.com> - e3887aa
- qubes-dom0-update: support 'list' and 'search' actions

* Thu Jan 30 2020 Marek Marczykowski-Górecki <marmarek@invisiblethingslab.com> - f98fb82
- dracut: install cirrus and bochs_drm into initramfs on relevant machines

* Wed Jan 15 2020 3hhh <tripleh@hackingthe.net> - c600b1b
- Proper argument escaping for special characters.

* Wed Jan 15 2020 3hhh <tripleh@hackingthe.net> - f4f5731
- Don't pass stdin to VMs unless necessary.

* Sat Jan 11 2020 Frédéric Pierret (fepitre) <frederic.pierret@qubes-os.org> - 0d53697
- travis: switch to dom0 Fedora 31

* Sat Jan 04 2020 Marek Marczykowski-Górecki <marmarek@invisiblethingslab.com> - a4006f5
- version 4.1.3

* Thu Dec 19 2019 Marek Marczykowski-Górecki <marmarek@invisiblethingslab.com> - 4a88c52
- kernel-install: consider both grub2 and grub2-efi configs

* Tue Dec 03 2019 Marta Marczykowska-Górecka <marta.maria.gorecka@gmail.com> - 761b5b1
- Added enabling of qrexec-policy-daemon.service

* Mon Oct 28 2019 Marek Marczykowski-Górecki <marmarek@invisiblethingslab.com> - 257d9e5
- version 4.1.2

* Mon Oct 21 2019 Marek Marczykowski-Górecki <marmarek@invisiblethingslab.com> - 9cf273d
- qubes-dom0-update: fix removing backup template after the operation

* Sat Oct 19 2019 Frédéric Pierret (fepitre) <frederic.pierret@qubes-os.org> - cf76a3c
- travis: switch to bionic

* Wed Oct 09 2019 Marek Marczykowski-Górecki <marmarek@invisiblethingslab.com> - e5e006d
- Fix various issues with qubes-dom0-update

* Tue Sep 24 2019 Marek Marczykowski-Górecki <marmarek@invisiblethingslab.com> - 2dadbcf
- version 4.1.1

* Wed Aug 28 2019 Marek Marczykowski-Górecki <marmarek@invisiblethingslab.com> - 9962fab
- repos rpc: use dnf native method of writing repository configuration

* Sun Jul 21 2019 M. Vefa Bicakci <m.v.b@runbox.com> - 1089a7a
- qubes-dom0-update: Quote arguments

* Thu Jun 27 2019 Marek Marczykowski-Górecki <marmarek@invisiblethingslab.com> - c56c4a7
- kernel-install: adjust EFI check to look for xen.cfg

* Sat Jun 08 2019 Marek Marczykowski-Górecki <marmarek@invisiblethingslab.com> - 895415a
- version 4.1.0

* Mon Apr 08 2019 AJ Jordan <alex@strugee.net> - 82806b5
- Add some comments to qubes.repos.List

* Mon Apr 08 2019 AJ Jordan <alex@strugee.net> - 3786197
- Don't write a trailing newline in qubes.repos.List

* Mon Apr 08 2019 AJ Jordan <alex@strugee.net> - 75faa22
- Add qubes.repos.* services to the RPMs

* Mon Apr 08 2019 AJ Jordan <alex@strugee.net> - 2283af8
- Print `ok` for repo enable/disable success

* Mon Apr 08 2019 AJ Jordan <alex@strugee.net> - 00c37b0
- Use qrexec service arguments

* Mon Apr 08 2019 AJ Jordan <alex@strugee.net> - 05658f0
- Properly set the umask for repo files

* Mon Apr 08 2019 AJ Jordan <alex@strugee.net> - 0af2769
- Enable/disable repos atomically

* Mon Apr 08 2019 AJ Jordan <alex@strugee.net> - 529f5a1
- Use Python whitespace conventions

* Mon Apr 08 2019 AJ Jordan <alex@strugee.net> - ce70209
- Rename admin.repos.* to qubes.repos.*

* Mon Apr 08 2019 AJ Jordan <alex@strugee.net> - 888073d
- Add admin.repos.* qrexec services

* Mon Apr 08 2019 Marek Marczykowski-Górecki <marmarek@invisiblethingslab.com> - 6fa3e19
- travis: drop R4.0, switch to xenial

* Mon Apr 08 2019 Marek Marczykowski-Górecki <marmarek@invisiblethingslab.com> - 15c55a4
- Remove qrexec related files

* Tue Apr 02 2019 Marek Marczykowski-Górecki <marmarek@invisiblethingslab.com> - 2ec29a4
- Cleanup lvm archived metadata files

* Tue Apr 02 2019 Marek Marczykowski-Górecki <marmarek@invisiblethingslab.com> - 330f155
- dom0-update: support rpm -K output of rpm 4.14

* Tue Apr 02 2019 Marek Marczykowski-Górecki <marmarek@invisiblethingslab.com> - fb1c284
- dom0-update: send dnf.conf to updatevm

* Mon Apr 01 2019 Marek Marczykowski-Górecki <marmarek@invisiblethingslab.com> - d705fa6
- system-config: enable dbus in system- and user- presets

* Sun Mar 31 2019 Marek Marczykowski-Górecki <marmarek@invisiblethingslab.com> - 61ec339
- qrexec: add version negotiation

* Sat Mar 09 2019 Marek Marczykowski-Górecki <marmarek@invisiblethingslab.com> - b6e3f36
- version 4.0.18

* Sat Mar 02 2019 Marek Marczykowski-Górecki <marmarek@invisiblethingslab.com> - edf406c
- travis: fix R4.1 line

* Sat Mar 02 2019 Marek Marczykowski-Górecki <marmarek@invisiblethingslab.com> - 1ffa0d6
- rpm: add BR: gcc

* Sat Mar 02 2019 Marek Marczykowski-Górecki <marmarek@invisiblethingslab.com> - d5e667d
- qvm-copy: fix handling VM names starting with dash

* Sun Feb 24 2019 Marek Marczykowski-Górecki <marmarek@invisiblethingslab.com> - e5deabe
- suspend: let errors be logged into journald

* Mon Feb 18 2019 Marek Marczykowski-Górecki <marmarek@invisiblethingslab.com> - 59ecf8e
- dom0-update: fix backup template name calculation

